﻿// ============================================================================
// Movo Network API Sample
//
// <ApiSample2.cs>
//
// Copyright (c) 2008 Servoland corporation.
// All rights reserved.
//
// 目的：
//		DDAを行うアプリケーションを作成します。
//
// Version情報：					  -- 追加, 修正項目 --
//		Version 1.0		2008.08.11	: First construction.
//
// ============================================================================
using System;
using MovoEtherVB.Servoland;


namespace CS_ApiSample2
{
    class CS_ApiSample2
    {
        private static MovoAPI api;
        private static int iDevSel;

        // =====================================================================
        // エントリーポイント
        // =====================================================================
        static void Main(string[] args)
        {
            int iRc;
            String line;

            iRc = Initialize();
            if (iRc != 0)
            {
                // 初期化失敗
                Console.WriteLine("Initialization failed (Hit Enter to exit)");
                line = Console.ReadLine();
                return;
            }

            ExecDda();

            Console.WriteLine("Finished (Hit Enter to exit)");
            line = Console.ReadLine();

        }

        // =====================================================================
        // 初期化
        // =====================================================================
        static int Initialize()
        {
            String strBuf = ""; // 文字列バッファ
            int i;              // Counter
            int iRc;            // Return code
            int IFNum;          // Ethernet I/Fの数
            int IFSel;          // 使用するEthetnet I/F
            String strIP = "";  // IPアドレス(文字列)
            int pIP = 0;          // IPアドレス(整数)
            String strSN = "";  // サブネットマスク(文字列)
            int pSN = 0;          // サブネットマスク(整数)

            api = new MovoAPI(); // MovoAPIクラスのインスタンス生成

            iDevSel = 0; // 通信デバイス=Ethernet

            // APIのバージョン取得
            api.GetLibVersion(ref strBuf);
            strBuf = "Network API version " + strBuf + "\n";

            Console.WriteLine(strBuf);

            // I/Fを設定(Ethernet使用時)
            if (iDevSel == 0)
            {
                // 使用可能なEthernet I/Fを検索
                IFNum = api.SearchInterface(iDevSel);
                if (IFNum < 0)
                {
                    Console.WriteLine("No ethernet I/F found");
                    return -1;
                }

                Console.WriteLine(IFNum.ToString() + " ethernet I/F found");

                // 各I/Fの情報を表示
                for (i = 0; i < IFNum; i++)
                {
                    api.GetHostIP(iDevSel, i, ref strIP, ref pIP);
                    api.GetHostSN(iDevSel, i, ref strSN, ref pSN);
                    Console.WriteLine(i.ToString() + " : " + String.Format("{0,-16}", strIP) + "(" + String.Format("{0,-16}", strSN) + ")");
                }

                if (IFNum == 1)
                {
                    // I/Fが１つのときは自動選択
                    IFSel = 0;
                }
                else
                {
                    // I/Fが２つ以上のときはユーザが選択
                    do
                    {
                        Console.Write("Select I/F to use [0-" + (IFNum - 1).ToString() + "]> ");
                        strBuf = Console.ReadLine();
                        try
                        {
                            IFSel = Convert.ToInt32(strBuf);
                        }
                        catch
                        {
                            IFSel = -1;
                        }
                    } while (IFSel < 0 || IFSel >= IFNum);

                }
                Console.WriteLine("I/F " + IFSel.ToString() + " selected\n");

                // Ethernet I/Fを設定
                api.SetInterface(iDevSel, IFSel);
            }

            // 接続されたドライバの検索
            iRc = api.SearchDevice(iDevSel, ref strBuf);
            if (iRc <= 0)
            {
                Console.WriteLine("No driver found");
                return -1;
            }

            // 接続可能なすべてのドライバに接続
            iRc = api.ConnectAll(iDevSel, ref strBuf);
            if (iRc > 0)
            {
                Console.WriteLine("Driver " + strBuf + " connected");
            }
            else
            {
                Console.WriteLine("No driver found");
                return -1;
            }
            return 0;
        }

        // =====================================================================
        // 特定コマンド待機
        //		（strComの返答がstrAnsと一致するまでPolling）
        // =====================================================================
        static int WaitCommand(char ID, String strCom, String strAns){
	        int iRc=0;
            String strBuf;

	        do{
                strBuf = ID.ToString() + ":" + strCom;
		        iRc=api.SendCom(iDevSel,ref strBuf);
                if (iRc == MovoAPI.MOVO_HASDATA)
                {
			        iRc=api.RecvCom(iDevSel,ref strBuf);
                }
                else if (iRc == MovoAPI.MOVO_NODATA)
                {
                    Console.WriteLine("Wrong usage");
			        return -1;
		        }else{
			        api.GetErrorText(iDevSel,ref strBuf);
                    Console.WriteLine(ID.ToString()+":Error [" + strBuf + "]");
			        return iRc;
		        }
	        }while(strBuf!=strAns);

	        return iRc;
        }

        // =============================================================================
        // DDA実行
        // =============================================================================
        static int ExecDda(){
	        const int DRV_NUM=2;
            const int DDA_LINE = 10000;
            const int size = MovoAPI.MOVO_MAX_DDA;	// 最大データ転送数

	        String strID="";		// ドライバの軸リスト
	        String strBuf;	    // 送受信バッファ
	        int flStart;		// 作動フラグ
	        int i;				// 汎用カウンタ
	        int j;				// DDA完了軸数カウンタ
	        int k;				// 汎用カウンタ
	        int Drv;			// 対象となるドライバの番号
	        int iRc;			// Return code
	        int data_num;		// データ個数
	        int all_fixed;		// 全軸完了フラグ
	        int send_num;			// 一回のデータ転送数
	        int[] finished=new int[DRV_NUM];	// DDA完了フラグ
	        int[] comp_num=new int[DRV_NUM];	// 転送済みデータ数
	        int[] rest_num=new int[DRV_NUM];	// 残りデータ数
	        int[] fix_num=new int [DRV_NUM];	// ログ取得数

            DDA_INFO CInfo=new DDA_INFO();		// DDA_INFO構造体
            DDA_DATA[,] DdaData=new DDA_DATA[DRV_NUM,DDA_LINE];	// DDAデータ
            DDA_DATA[,] LogData=new DDA_DATA[DRV_NUM,DDA_LINE+1];	// Logデータ
            DDA_DATA dummydata=new DDA_DATA();

            String filename="log.csv";
            System.IO.StreamWriter logfile;
        	
	        iRc=api.GetConnectedDeviceInfo(iDevSel,ref strID);
	        if(iRc<DRV_NUM){
                Console.WriteLine("Not enough drivers found");
		        return -1;
	        }

	        // テスト用ＤＤＡデータ作成
	        //（モータの動きが観測できる程度の尺度にする）
	        for(i=0;i<DDA_LINE;i++){
		        for(Drv=0;Drv<DRV_NUM;Drv++){
                    DdaData[Drv, i].data32 = (int)(1000000 * Math.Sin(2 * Math.PI * 5 * i / DDA_LINE));
                    DdaData[Drv, i].trj32 = (uint)i;
		        }
	        }

	        flStart=0;			// 作動フラグ初期化
	        all_fixed=0;		// 全軸完了フラグ初期化
	        data_num=DDA_LINE;	// DDAデータ行数初期化

	        for(Drv=0;Drv<DRV_NUM;Drv++){
		        finished[Drv]=0;		// DDA完了フラグ初期化
		        comp_num[Drv]=0;		// 転送済みデータ数初期化
		        rest_num[Drv]=data_num;	// 残りデータ数初期化
		        fix_num[Drv]=0;			// ログ取得数初期化
	        }

	        for(Drv=0;Drv<DRV_NUM;Drv++){
		        if(Drv==0){
			        // 一台をMasterに設定
                    strBuf = strID[Drv].ToString() + ":#70=1";
		        }else{
			        // 残りをSlaveに設定
                    strBuf = strID[Drv].ToString() + ":#70=2";
		        }
		        iRc=api.SendCom(iDevSel,ref strBuf);
		        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;
	        }

	        // SlaveのSYNCSが1になるのを待つ(Masterは常に0)
	        iRc=WaitCommand(strID[1],"SYNCS","1");
            strBuf = "*:MASK";
	        iRc=api.SendCom(iDevSel,ref strBuf);
            if (iRc == MovoAPI.MOVO_FAILURE) goto COM_ERR;

	        // サーボON
            strBuf="*:SVON";
	        iRc=api.SendCom(iDevSel,ref strBuf);
	        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;

	        // Wait for SVON
	        for(Drv=0;Drv<DRV_NUM;Drv++){
		        iRc=WaitCommand(strID[Drv],"SVONS","1");
		        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;
	        }

	        // DDAモードに設定
            strBuf="*:DDA";
	        iRc=api.SendCom(iDevSel,ref strBuf);
	        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;

	        // LINK開始
            strBuf="*:LINK=1";
	        iRc=api.SendCom(iDevSel,ref strBuf);
	        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;

	        // DDA初期化
	        for(Drv=0;Drv<DRV_NUM;Drv++){
		        iRc=api.InitDda(
			        iDevSel,	// Device select
			        strID[Drv],	// 軸設定
			        0,			// 1ms サンプリング
			        1,			// 指令値とログ用
			        1,			// feed forward on
			        ref CInfo);	// DDA_INFO構造体へのポインタ
		        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;
	        }

	        // DDAデータ設定
	        Drv=0;
	        for(;;){
		        // 完了フラグ判定
		        if(finished[Drv]==1){
			        if(++Drv>=DRV_NUM)
				        Drv=0;
			        continue;
		        }
		        // １回に送るデータ数を設定
		        if(rest_num[Drv]==0){			// 残りデータ数なし
			        send_num=0;
			        all_fixed++;
		        }else if(rest_num[Drv]>size){	// 残りデータ数が最大転送数を上回る
			        send_num=size;
		        }else{
			        send_num=rest_num[Drv];
		        }

		        if(all_fixed==DRV_NUM){
			        // 全軸バッファへのデータ転送完了
			        for(k=0;k<DRV_NUM;k++){
				        strBuf = strID[k].ToString() + ":G";
				        iRc=api.SendCom(iDevSel, ref strBuf);	// 作動
				        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;
			        }
			        flStart=1;	// 作動フラグ設定
		        }

		        // DDAデータを設定
                if (flStart == 1)
                {
                    if (send_num == 0){
                        iRc = api.SetDda(iDevSel, strID[Drv], send_num, ref dummydata, ref LogData[Drv, fix_num[Drv]], ref CInfo);
                    }
                    else
                    {
                        iRc = api.SetDda(iDevSel, strID[Drv], send_num, ref DdaData[Drv, comp_num[Drv]], ref LogData[Drv, fix_num[Drv]], ref CInfo);
                    }
                }
                else
                    iRc = api.SetDda(iDevSel, strID[Drv], send_num, ref DdaData[Drv, comp_num[Drv]], ref dummydata, ref CInfo);
                if (iRc == MovoAPI.MOVO_FAILURE) goto COM_ERR;

		        // ログ数加算
		        if(flStart==1)
			        fix_num[Drv]+=CInfo.log_num;

		        // Busyフラグチェック
		        if(CInfo.busy==1){	// ドライバのＤＤＡ領域が一杯
			        if(flStart==0){
				        // 対象軸のバッファへのデータ転送完了
				        for(k=0;k<DRV_NUM;k++){
                            strBuf = strID[k].ToString() + ":G";
					        iRc=api.SendCom(iDevSel, ref strBuf);	// 作動
					        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;
				        }
				        flStart=1;	// 作動フラグ設定
			        }else{
				        // バッファが一杯
                        System.Threading.Thread.Sleep(10);	// 待機
				        if(++Drv>=DRV_NUM)	// 軸変更
					        Drv=0;
			        }
		        }else if(fix_num[Drv]>=data_num && CInfo.status==1){
			        // ロギング完了
			        finished[Drv]=1;
			        LogData[Drv,data_num].data32=(int)CInfo.enc;
			        LogData[Drv,data_num].trj32=(uint)CInfo.trj;
			        if(++Drv>=DRV_NUM)
				        Drv=0;
        			
			        // DDA完了軸数取得
			        for(i=0,j=0;i<DRV_NUM;i++)
				        j+=finished[Drv];
			        if(j>=DRV_NUM)
				        break;
			        else
				        continue;	// DDA完了
		        }else{
			        rest_num[Drv]-=send_num;	// 残りデータ算出
			        comp_num[Drv]+=send_num;	// 転送完了データ算出
			        if(++Drv>=DRV_NUM){
				        all_fixed=0;
				        Drv=0;
			        }
		        }
	        }

	        // DDA停止
            strBuf="*:S";
	        iRc=api.SendCom(iDevSel,ref strBuf);
	        if(iRc==MovoAPI.MOVO_FAILURE)goto COM_ERR;

	        // ベリファイ(軌道番号比較)
	        for(i=0;i<DDA_LINE;i++){
		        for(Drv=0;Drv<DRV_NUM;Drv++){
			        // 注：LogDataの先頭に格納されるデータは
			        // 前回DDAの最終データに対するEncoder値で、今回のDDA結果ではない。
			        if(DdaData[Drv,i].trj32!=LogData[Drv,i+1].trj32){
				        Console.WriteLine("Verification failed");
				        return -1;
			        }
		        }
	        }

            // ログ書き出し
            strBuf = "";
            logfile = new System.IO.StreamWriter(filename);
            for(i = 0;i<DDA_LINE;i++){
                for(Drv = 0;Drv<DRV_NUM;Drv++){
                    strBuf += DdaData[Drv, i].trj32.ToString() + "," + DdaData[Drv, i].data32.ToString() + "," + LogData[Drv, i].data32.ToString();
                    if(Drv!= DRV_NUM - 1){
                        strBuf += ",";
                    }
                }
                logfile.WriteLine(strBuf);
                strBuf = "";
            }
            logfile.Close();

            return 0;

        COM_ERR:
	        // 通信エラー
	        api.GetErrorText(iDevSel, ref strBuf);
            strBuf="Error : " + strBuf;
            Console.WriteLine(strBuf);
	        return -1;
        }
    
    
    }
}
