﻿' ============================================================================
' Movo Network API Sample
'
' <ApiSample2.vb>
'
' Copyright (c) 2008 Servoland corporation.
' All rights reserved.
'
' 目的：
'		DDAを行うアプリケーションを作成します。
'
' Version情報：					    -- 追加, 修正項目 --
'		Version 1.0		2008.08.11	: First construction.
'
' ============================================================================
Imports MovoEtherVB.Servoland

Module VB_ApiSample2
    Dim iDevSel As Integer
    Dim api As MovoAPI

    ' =========================================================================
    ' エントリーポイント
    ' =========================================================================
    Sub Main()
        Dim iRc As Integer
        Dim line As String

        iRc = Initialize()
        If iRc <> 0 Then
            ' 初期化失敗
            Console.WriteLine("Initialization failed (Hit Enter to exit)")
            line = Console.ReadLine()
            Return
        End If

        ExecDda()

        Console.WriteLine("Finished (Hit Enter to exit)")
        line = Console.ReadLine()

    End Sub

    ' =========================================================================
    ' 初期化
    ' =========================================================================
    Function Initialize() As Integer
        Dim Str As String = ""  ' 文字列バッファ
        Dim i As Integer        ' Counter
        Dim iRc As Integer      ' Return code
        Dim IFNum As Integer    ' Ethernet I/Fの数
        Dim IFSel As Integer    ' 使用するEthetnet I/F
        Dim strIP As String = ""    ' IPアドレス(文字列)
        Dim pIP As Integer          ' IPアドレス(整数)
        Dim strSN As String = ""    ' サブネットマスク(文字列)
        Dim pSN As Integer          ' サブネットマスク(整数)

        api = New MovoAPI ' MovoAPIクラスのインスタンス生成

        iDevSel = 0 ' 通信デバイス=Ethernet

        ' APIのバージョン取得
        api.GetLibVersion(Str)
        Str = "Network API version " + Str + ControlChars.NewLine

        Console.WriteLine(Str)

        ' I/Fを設定(Ethernet使用時)
        If iDevSel = 0 Then
            ' 使用可能なEthernet I/Fを検索
            IFNum = api.SearchInterface(iDevSel)
            If IFNum < 0 Then
                Console.WriteLine("No ethernet I/F found")
                Return -1
            End If
            Console.WriteLine(IFNum.ToString + " ethernet I/F found")

            ' 各I/Fの情報を表示
            For i = 0 To IFNum - 1
                api.GetHostIP(iDevSel, i, strIP, pIP)
                api.GetHostSN(iDevSel, i, strSN, pSN)
                Console.WriteLine(i.ToString + " : " + String.Format("{0,-16}", strIP) + "(" + String.Format("{0,-16}", strSN) + ")")
            Next

            If IFNum = 1 Then
                ' I/Fが１つのときは自動選択
                IFSel = 0
            Else
                ' I/Fが２つ以上のときはユーザが選択
                Do
                    Console.Write("Select I/F to use [0-" + (IFNum - 1).ToString + "]> ")
                    Str = Console.ReadLine()
                    Try
                        IFSel = Convert.ToInt32(Str)
                    Catch
                        IFSel = -1
                    End Try
                Loop While IFSel < 0 Or IFSel >= IFNum

            End If
            Console.WriteLine("I/F " + IFSel.ToString + " selected" + ControlChars.NewLine)

            ' Ethernet I/Fを設定
            api.SetInterface(iDevSel, IFSel)

        End If

        ' 接続されたドライバの検索
        iRc = api.SearchDevice(iDevSel, Str)
        If iRc <= 0 Then
            Console.WriteLine("No driver found")
            Return -1
        End If

        ' 接続可能なすべてのドライバに接続
        iRc = api.ConnectAll(iDevSel, Str)
        If iRc > 0 Then
            Console.WriteLine("Driver " + Str + " connected")
        Else
            Console.WriteLine("No driver found")
            Return -1
        End If

        Return 0

    End Function

    ' =========================================================================
    ' 特定コマンド待機
    '		（strComの返答がstrAnsと一致するまでPolling）
    ' =========================================================================
    Function WaitCommand(ByVal ID As Char, ByVal strCom As String, ByVal strAns As String) As Integer
        Dim iRc As Integer
        Dim strBuf As String

        Do
            strBuf = ID.ToString + ":" + strCom
            iRc = api.SendCom(iDevSel, strBuf)
            If iRc = MovoAPI.MOVO_HASDATA Then
                iRc = api.RecvCom(iDevSel, strBuf)
            ElseIf iRc = MovoAPI.MOVO_NODATA Then
                Console.WriteLine("Wrong usage")
                Return -1
            Else
                api.GetErrorText(iDevSel, strBuf)
                Console.WriteLine(ID.ToString + ":Error [" + strBuf + "]")
                Return iRc
            End If
        Loop While strBuf <> strAns

        Return iRc
    End Function

    ' =========================================================================
    ' DDA実行
    ' =========================================================================
    Function ExecDda() As Integer
        Const DRV_NUM As Integer = 2
        Const DDA_LINE As Integer = 10000
        Const size As Integer = MovoAPI.MOVO_MAX_DDA

        Dim strID As String = ""
        Dim strBuf As String
        Dim flStart As Integer
        Dim i, j, k As Integer
        Dim Drv As Integer
        Dim iRc As Integer
        Dim data_num As Integer
        Dim all_fixed As Integer
        Dim send_num As Integer
        Dim finished(DRV_NUM) As Integer
        Dim comp_num(DRV_NUM) As Integer
        Dim rest_num(DRV_NUM) As Integer
        Dim fix_num(DRV_NUM) As Integer

        Dim CInfo As DDA_INFO
        Dim DdaData(DRV_NUM, DDA_LINE) As DDA_DATA
        Dim LogData(DRV_NUM, DDA_LINE + 1) As DDA_DATA
        Dim dummydata As DDA_DATA

        Dim filename As String = "log.csv"
        Dim logfile As IO.StreamWriter

        iRc = api.GetConnectedDeviceInfo(iDevSel, strID)
        If iRc < DRV_NUM Then
            Console.WriteLine("Not enough drivers found\n")
            Return -1
        End If

        ' テスト用ＤＤＡデータ作成
        '（モータの動きが観測できる程度の尺度にする）
        For i = 0 To DDA_LINE - 1
            For Drv = 0 To DRV_NUM - 1
                DdaData(Drv, i).data32 = 1000000 * Math.Sin(2.0 * Math.PI * 5.0 * i / DDA_LINE)
                DdaData(Drv, i).trj32 = i
            Next
        Next

        flStart = 0             ' 作動フラグ初期化
        all_fixed = 0           ' 全軸完了フラグ初期化
        data_num = DDA_LINE     ' DDAデータ行数初期化

        For Drv = 0 To DRV_NUM - 1
            finished(Drv) = 0           ' DDA完了フラグ初期化
            comp_num(Drv) = 0           ' 転送済みデータ数初期化
            rest_num(Drv) = data_num    ' 残りデータ数初期化
            fix_num(Drv) = 0            ' ログ取得数初期化
        Next

        For Drv = 0 To DRV_NUM - 1
            If Drv = 0 Then
                ' 一台をMasterに設定
                strBuf = strID(Drv).ToString + ":#70=1"
            Else
                ' 残りをSlaveに設定
                strBuf = strID(Drv).ToString + ":#70=2"
            End If
            iRc = api.SendCom(iDevSel, strBuf)
            If iRc = MovoAPI.MOVO_FAILURE Then
                GoTo COM_ERR
            End If
        Next

        ' SlaveのSYNCSが1になるのを待つ(Masterは常に0)
        iRc = WaitCommand(strID(1), "SYNCS", "1")

        iRc = api.SendCom(iDevSel, "*:MASK")
        If iRc = MovoAPI.MOVO_FAILURE Then
            GoTo COM_ERR
        End If

        ' サーボON
        iRc = api.SendCom(iDevSel, "*:SVON")
        If iRc = MovoAPI.MOVO_FAILURE Then
            GoTo COM_ERR
        End If

        ' Wait for SVON
        For Drv = 0 To DRV_NUM - 1
            iRc = WaitCommand(strID(Drv), "SVONS", "1")
            If iRc = MovoAPI.MOVO_FAILURE Then
                GoTo COM_ERR
            End If
        Next

        ' DDAモードに設定
        iRc = api.SendCom(iDevSel, "*:DDA")
        If iRc = MovoAPI.MOVO_FAILURE Then
            GoTo COM_ERR
        End If

        ' LINK開始
        iRc = api.SendCom(iDevSel, "*:LINK=1")
        If iRc = MovoAPI.MOVO_FAILURE Then
            GoTo COM_ERR
        End If

        ' DDA初期化
        For Drv = 0 To DRV_NUM - 1
            iRc = api.InitDda(iDevSel, strID(Drv), 0, 1, 1, CInfo)
            If iRc = MovoAPI.MOVO_FAILURE Then
                GoTo COM_ERR
            End If
        Next


        ' DDAデータ設定
        Drv = 0
        Do
            ' 完了フラグ判定
            If finished(Drv) Then
                Drv += 1
                If Drv >= DRV_NUM Then
                    Drv = 0
                End If
                Continue Do
            End If
            ' １回に送るデータ数を設定
            If rest_num(Drv) = 0 Then ' 残りデータ数なし
                send_num = 0
                all_fixed += 1
            ElseIf rest_num(Drv) > size Then ' 残りデータ数が最大転送数を上回る
                send_num = size
            Else
                send_num = rest_num(Drv)
            End If

            If all_fixed = DRV_NUM Then
                ' 全軸バッファへのデータ転送完了
                For k = 0 To DRV_NUM - 1
                    strBuf = strID(k).ToString + ":G"
                    iRc = api.SendCom(iDevSel, strBuf) ' 作動
                    If iRc = MovoAPI.MOVO_FAILURE Then
                        GoTo COM_ERR
                    End If
                Next
                flStart = 1 ' 作動フラグ設定
            End If

            ' DDAデータを設定
            If flStart Then
                If send_num = 0 Then
                    iRc = api.SetDda(iDevSel, strID(Drv), send_num, dummydata, LogData(Drv, fix_num(Drv)), CInfo)
                Else
                    iRc = api.SetDda(iDevSel, strID(Drv), send_num, DdaData(Drv, comp_num(Drv)), LogData(Drv, fix_num(Drv)), CInfo)
                End If
            Else
                iRc = api.SetDda(iDevSel, strID(Drv), send_num, DdaData(Drv, comp_num(Drv)), dummydata, CInfo)
            End If

            If iRc = MovoAPI.MOVO_FAILURE Then
                GoTo COM_ERR
            End If

            ' ログ数加算
            If flStart Then
                fix_num(Drv) += CInfo.log_num
            End If

            ' Busyフラグチェック
            If CInfo.busy = 1 Then ' ドライバのＤＤＡ領域が一杯
                If flStart = 0 Then
                    ' 対象軸のバッファへのデータ転送完了()
                    For k = 0 To DRV_NUM - 1
                        strBuf = strID(k).ToString + ":G"
                        iRc = api.SendCom(iDevSel, strBuf) ' 作動
                        If iRc = MovoAPI.MOVO_FAILURE Then
                            GoTo COM_ERR
                        End If
                    Next
                    flStart = 1 ' 作動フラグ設定
                Else
                    ' バッファが一杯
                    System.Threading.Thread.Sleep(10) ' 待機
                    Drv += 1
                    If Drv >= DRV_NUM Then ' 軸変更
                        Drv = 0
                    End If
                End If
            ElseIf fix_num(Drv) >= data_num And CInfo.status = 1 Then
                ' ロギング完了
                finished(Drv) = 1
                LogData(Drv, data_num).data32 = CType(CInfo.enc, Integer)
                'LogData(Drv, data_num).data32 = CInfo.enc
                LogData(Drv, data_num).trj32 = CInfo.trj
                Drv += 1
                If Drv >= DRV_NUM Then
                    Drv = 0
                End If

                ' DDA完了軸数取得
                j = 0
                For i = 0 To DRV_NUM - 1
                    j += finished(Drv)
                Next
                If j >= DRV_NUM Then
                    Exit Do
                Else
                    Continue Do
                End If
            Else
                rest_num(Drv) -= send_num ' 残りデータ算出
                comp_num(Drv) += send_num ' 転送完了データ算出
                Drv += 1
                If Drv >= DRV_NUM Then
                    all_fixed = 0
                    Drv = 0
                End If
            End If
        Loop

        ' DDA停止
        iRc = api.SendCom(iDevSel, "*:S")
        If iRc = MovoAPI.MOVO_FAILURE Then
            GoTo COM_ERR
        End If

        ' ベリファイ(軌道番号比較)
        For i = 0 To DDA_LINE - 1
            For Drv = 0 To DRV_NUM - 1
                ' 注：LogDataの先頭に格納されるデータは
                ' 前回DDAの最終データに対するEncoder値で、今回のDDA結果ではない。
                If DdaData(Drv, i).trj32 <> LogData(Drv, i + 1).trj32 Then
                    Console.WriteLine("Verification failed")
                    Return -1
                End If
            Next
        Next

        ' ログ書き出し
        strBuf = ""
        logfile = New IO.StreamWriter(filename)
        For i = 0 To DDA_LINE - 1
            For Drv = 0 To DRV_NUM - 1
                strBuf += DdaData(Drv, i).trj32.ToString + "," + DdaData(Drv, i).data32.ToString + "," + LogData(Drv, i).data32.ToString
                If (Drv <> DRV_NUM - 1) Then
                    strBuf += ","
                End If
            Next
            logfile.WriteLine(strBuf)
            strBuf = ""
        Next
        logfile.Close()

        Return 0

COM_ERR:
        ' 通信エラー
        strBuf = ""
        api.GetErrorText(iDevSel, strBuf)
        Console.WriteLine("Error : " + strBuf)
        Return -1

    End Function

End Module
